type expr =
	| Const of float (* constante *)
	| Add   of expr * expr (* Add(e1, e2) correspond à e1 + e2 *)
	| Mul   of expr * expr (* Mul(e1, e2) correspond à e1 * e2 *)
	| Var   of string

type context = (string * float) list

let rec get_var (s: string) (c: context) : float =
	match c with 
	| [] -> failwith "Variable pas présente"
	| (v, a)  :: q -> 
		if s = v then a 
		else get_var s q

(* valeur de e interprétée dans ctx *)
let rec eval (e: expr) (ctx: context) : float = 
	match e with 
	| Const x -> x 
	| Var v -> get_var v ctx
	| Add (e1, e2) -> 
		let v1 = eval e1 ctx in 
		let v2 = eval e2 ctx in 
		v1 +. v2
	| Mul (e1, e2) -> 
		let v1 = eval e1 ctx in 
		let v2 = eval e2 ctx in 
		v1 *. v2

(* Test: évaluer (x + 2) * (y + (x * 5)) 
   avec x = -0.5, y = 2*)

let c1 = [("x", -0.5); ("y", 2.0)]  

let e =
Mul(
	Add(
		Var "x",
		Const 2.
	),
	Add(
		Var "y",
		Mul(
			Var "x",
			Const 5.
		)	
	)
)

let test () =
	assert (eval e c1 = -0.75)